$MOD51
;mods for 89S ISP programming
;Friday, 04 June, 2010, 9:08:52 AM
;reads,erases, writes perfectly
;
;lock works but PC program says it failed
;because the 89S series devices do not read FF when locked
;they read 0001020304..etc
;
;fast verify works now! (when writing, checksum must be enabled)
;-------------------------------------------------
cr	EQU	0dh	;Return
lf	EQU	0ah	;line feed
space	EQU	20h	;ascii space
XON 	EQU	11h
XOFF 	EQU	13h
;
;adapted for AVR910 Board @ 11.0592 MHz
LED_Grn	equ	p1.1		;done, or success LED (ON = LOW)
LED_Yel	equ	p1.2		;busy LED (ON = LOW)
LED_Red	equ	p1.3		;error LED (ON = LOW)
RST		equ	p1.4		;may need 10K pullup on this pin 
					;depending on which C51 device you are using
MOSI		equ	p1.5		;
MISO		equ	p1.6		;
SCK		equ	p1.7		;
;
time1		equ	70h
time2		equ	71h
time3		equ	72h
bytecount	equ	73h
bitcount	equ	74h
dyta		equ	75h
saveDPH	equ	76h
saveDPL	equ	77h
;
Flags       DATA  20h         ; Error Flags
ErrFlag     BIT   Flags.0     ; set if Enable Programming Error
;
org	00000h
;--------------------------------------------
	jmp	Start			;

Start:
	MOV	SP,#007H
	mov	a,#0ffh		;clear the ports 
	mov	p1,a			;
	mov	p3,a			;
	
	setb	MOSI			;
	setb	MISO			;
	clr	SCK			;
	clr	RST			;	
	mov	Flags,#0
;--------------------------------------------
Main:
	clr	a			;
	mov	i_high,a		;
	mov	i_low,a		;
	clr	a			;
	mov	countH,a		;
	mov	countL,a		;
	mov	scon,#052h		;8-bit UART mode
	mov	tmod,#020h		;timer 1 mode 2 auto reload
	mov	th1,#0fdh		;9600 8n1
	setb	tr1			;run timer1
	call	getchar		;get char to invoke title printing, ignore char

	call	En_pg_Mode		;
	jb	ErrFlag,Stop	;endless loop if error entering prog mode

	mov	dptr,#title		;print string
	call	putstr		;"Easy-Downloader V2.0 for ATMEL 89S51/52 (sdcc version)"

	call	sendprompt		;
	jmp	MainLoop
;--------------------------------------------
stop:
	jmp	stop
;--------------------------------------------
MainLoop:
	call	getcommand		;
	call	prompting		;
	call	setcounter		;
	call	erase			;
	call	write			;
	call	read			;
	call	lock			;
	call	printchksum		;need this for fast verify
;	call	getinfo		;
	call	PGMparameters	;
	jmp	MainLoop		;
;--------------------------------------------
erase:
	mov	a,command		;
	cjne	a,#065h,eraseRET	;if command = 'e'

	call	En_pg_Mode		;
	jb	ErrFlag,Stop	;endless loop if error entering prog mode

	call	erase_isp		;
	call	putok			;

eraseRET:
	ret				;
;--------------------------------------------
;call sequence before write: 's', 'p','e','w' 
write:
	mov	a,command		;
	cjne	a,#077h,NotWCmd	;if command = 'w'
	jmp	write1		;

NotWCmd:
	jmp	writeRET		;

Write1:
	call	En_pg_Mode		;
	jb	ErrFlag,Stop	;endless loop if error entering prog mode

	clr	LED_Yel

	clr	a			;
	mov	chksumH,a		;
	mov	chksumL,a		;
	mov	i_high,a		;
	mov	i_low,a		;
	
WriteLoop:
	mov	r2,i_low		;
	mov	r3,i_high		;

	clr	c			;
	mov	a,r2			;
	subb	a,countL		;
	mov	a,r3			;
	subb	a,countH		;
	jnc	Write_OK		;

	mov	dpl,#xon		;ask Console for a char
	call	putchar		;

	mov	addressL,i_low	;
	mov	addressH,i_high	;

	call	GetChr		;get char but don't echo char to host
	mov	dyta,dpl		;

	mov	dpl,#xoff		;tell Console we are busy
	call	putchar		;

	mov a, #040h		; SPI write (byte 1)
	call SPI 			;
	mov a,addressH		; SPI write (byte 2)
	call SPI 			;
	mov a,addressL		; SPI write (byte 3)
	call SPI 			;
	mov a, dyta			; SPI write (byte 4)
	call SPI 			;
	call	time1mS		; delay 1mS

	mov	r2,dyta		;
	mov	r3,#000h		;
	mov	a,r2			;
	add	a,chksumL		;
	mov	chksumL,a		;
	mov	a,r3			;
	addc	a,chksumH		;
	mov	chksumH,a		;summing chksum 

Write3:
	inc	i_low			;
	clr	a			;
	cjne	a,i_low,Write4	;
	inc	i_high		;

Write4:
	jmp	WriteLoop		;

Write_OK:
	setb	LED_Yel		;LED OFF
	call	putok			;

WriteRet:
	ret				;
;-------------------------------------------
;read according to number of bytes by 's' command  
read:
	mov	a,command		;
	cjne	a,#072h,NotrCmd	;
	jmp	Read1			;

NotrCmd:
	jmp	ReadRet		;

Read1:
	clr	LED_Yel		;LED on
	clr	a			;
	mov	chksumH,a		;reset check sum word 
	mov	chksumL,a		;

	mov	blank,#031h		;
	mov	r2,#000h		;
	mov	r3,#000h		;

ReadLoop:
	clr	c			;
	mov	a,r2			;
	subb	a,countL		;
	mov	a,r3			;
	subb	a,countH		;
	jnc	ReadEnd		;

	mov	addressL,r2		;
	mov	addressH,r3		;

	mov a, #020h		; SPI write (byte 1)
	call SPI 			;
	mov a, addressH		; SPI write (byte 2)
	call SPI 			;
	mov a, addressL		; SPI write (byte 3)
	call SPI 			;		
	mov a, #0FFh		; send dummy byte	
	call SPI 			; SPI read (byte 4)
	mov	R4,a			; save data in R4

	mov	r5,#000h		;do checksum
	mov	a,r4			;
	add	a,chksumL		;
	mov	chksumL,a		;
	mov	a,r5			;
	addc	a,chksumH		;save new checksum
	mov	chksumH,a		;

	mov	dpl,r4		;
	push	002h			;
	push	003h			;
	call	puthex		;print in HEX 
	pop	003h			;
	pop	002h			;

	push	002h			;
	push	003h			;
	call	chkXOFF		;
	pop	003h			;
	pop	002h			;

	inc	r2			;inc addrlo
	cjne	r2,#000h,Read2	;
	inc	r3			;inc addrhi

Read2:
	jmp	ReadLoop		;

ReadEnd:
	setb	LED_Yel		;LED OFF
	call	putok			;

ReadRet:
	ret				;
;--------------------------------------------
;if all bytes are 0FFH then blank =FF, else blank = 0 
blanktest:
	mov	a,R4			;
	cjne	a,#0ffh,Notblank	;
	jmp	blanktestRET	;
;
Notblank:
	mov	blank,#030h		;count how many bytes are not blank
	inc	nonblankL		;
	clr	a			;
	cjne	a,nonblankL,blanktestRET ;
	inc	nonblankH		;

blanktestRET:
	ret				;
;---------------------------------------------
;only protection mode 4 can be programmed, i.e., disabled further
;program, verify, and external execution
;
lock:
	mov	a,command		;
	cjne	a,#06ch,lockRET	;if command = 'l'

	call	Lock_ISP		;
	call	putok			;

lockRET:
	ret				;
;--------------------------------------------
printchksum:
	mov	a,command		;if command = 'c'
	cjne	a,#063h,printchksumRET;

	mov	dptr,#str_0		;
	call	putstr		;

	mov	a,chksumL		;
	cpl	a			;
	mov	r2,a			;
	mov	a,chksumH		;
	cpl	a			;
	mov	r3,a			;
	mov	a,#001h		;
	add	a,r2			;
	mov	dpl,a			;
	clr	a			;
	addc	a,r3			;
	mov	dph,a			;
	call	puthex16		;
	call	putok			;

printchksumRET:
	ret				;
;--------------------------------------------
;read signature ATMEL chip 51/52 Vpp only 5V 
signature:
	call	read_sig		;
	ret				;
;-----------------------------------------------------
testblank:
	call	signature		;
	mov	a,chip		;
	cjne	a,#051h,L060F	;89C51 detected
	mov	bytesL,#000h	;if chip = 0x51 then bytes = 4kB 
	mov	bytesH,#010h	;
	jmp	L066E			;
;
L060F:
	mov	a,chip		;
	cjne	a,#052h,L0669	;89C52 detected
	mov	bytesL,#000h	;if chip = 0x52 then bytes = 8kB 
	mov	bytesH,#020h	;
	jmp	L066E			;

L0669:
	clr	a			;invalid chip
	mov	bytesH,a		;erase bytes counter
	mov	bytesL,a		;
;
L066E:
	clr	a			;reset check sum word 
	mov	chksumH,a		;
	mov	chksumL,a		;
	clr	a			;reset nonblank bytes counter
	mov	nonblankH,a		;
	mov	nonblankL,a		;

	mov	blank,#031h		;
	mov	r2,#000h		;
	mov	r3,#000h		;

;don't dump the whole flash, we don't care if its blank or not
;because we are erasing it before programming!
testblankLoopRET:
	ret				;
;--------------------------------------------
setcounter:
	mov	a,command		;if command = 's'
	cjne	a,#073h,setcounterRET;

	call	getnum		;
	mov	r2,dpl		;get hex word in r3:r2
	mov	r3,dph		;
	mov	countL,r2		;save it to countH:countL
	mov	countH,r3		;
	call	putok			;

setcounterRET:
	ret				;
;--------------------------------------------
;getinfo:
;	mov	a,command		;if command ='g'
;	cjne	a,#067h,NotCmdg;
;	jmp	L06F5			;
;
;NotCmdg:
;	jmp	getinfoRET		;
;
;L06F5:
;	call	testblank		;
;	mov	a,chip		;
;	cjne	a,#0ffh,L0700	;
;	jmp	L077E			;
;
;L0700:
;	mov	dptr,#str_1		;print string "found target"
;	call	putstr		;
;	mov	dpl,chip		;
;	call	puthex		;send chip ID
;	mov	a,vpp			;
;	cjne	a,#0ffh,L0729	;not high voltage programming?
;
;	mov	dptr,#str_2		;print string ("-12V")
;	call	putstr		;
;
;L0729:
;	mov	a,vpp			;
;	cjne	a,#006h,L073D	;if vpp = 6 (3rd sig byte)
;
;	mov	dptr,#str_3		;print string "-5V"
;	call	putstr		;
;
;L073D:
;	mov	dptr,#str_4		;print string "nonblank"
;	call	putstr		;
;	mov	dpl,nonblankL	;send nonblank bytes word to console
;	mov	dph,nonblankH	;
;	call	putnum		;
;
;	mov	dptr,#str_5		;print string "bytes"
;	call	putstr		;
;
;	mov	dptr,#str_6		;print string "bytes counter"
;	call	putstr		;
;	mov	dpl,countL		;
;	mov	dph,countH		;
;	call	putnum		;
;	jmp	L078D			;
;
;L077E:
;	mov	dptr,#str_7		;print string "Not found 89Sxx"
;	call	putstr		;
;
;L078D:
;	call	putok			;
;
;getinfoRET:
;	ret				;
;--------------------------------------------
;for simple host interfacing
PGMparameters:
	mov	a,command		;if command = 'p'
	cjne	a,#070h,PGMparametersRET;

	call	testblank		;
	mov	dpl,chip		;
	call	puthex		;

	mov	dpl,#','
	call	putchar		;
	mov	dpl,nonblankL	;
	mov	dph,nonblankH	;
	call	putnum		;

	mov	dpl,#','
	call	putchar		;

	mov	dpl,countL		;
	mov	dph,countH		;
	call	putnum		;

	call	putok			;

PGMparametersRET:
	ret				;
;--------------------------------------------
$include(strings.inc)
$include(math.inc)
$include(spi.inc)
$include(delay.inc)
$include(uart.inc)
;--------------------------------------------
DSEG
org 0030h
i_low:	ds 1		;30h
i_high:	ds 1		;31h
;
accu:		ds 1		;32h ;this one not used
temp:		ds 1		;33h
;
blank:	ds 1		;34h
chip:		ds 1		;35h
vpp:		ds 1		;36h
command:	ds 1		;37h
addressL:	ds 1		;38h
addressH:	ds 1		;39h
chksumL:	ds 1		;3Ah
chksumH:	ds 1		;3Bh

countL:	ds 1		;3Ch
countH:	ds 1		;3Dh
bytesL:	ds 1		;3Eh
bytesH:	ds 1		;3Fh
nonblankL:	ds 1		;40h
nonblankH:	ds 1		;41h
;
getnum_s_1_1: 	ds 6	;42h
putnum_s_1_1: 	ds 6	;48h
;
;org 4EH
muluint_PARM2L: 	ds 1	;4Eh
muluint_PARM2H: 	ds 1	;4Fh
;--------------------------------------------
END