;***************************************************************
; Prepare the target AT89S52 for programming
; ErrFlag Set if error occurs
En_pg_Mode:
	clr	ErrFlag	;
	mov	bytecount,#32	;	

	clr	SCK		;
	clr	RST
	lcall	Delay50mS	; wait 50 ms (arbitrary)

	setb RST 		; force target into reset 		
	lcall	Delay50mS	; wait 50 ms (arbitrary)
En_pg:
	mov a, #0ACh	; send first byte of enable code
	lcall SPI 		;

	mov a, #053h	; send second byte
	lcall SPI 		;

	mov a, #0FFh	; send third byte as dummy
	lcall SPI 		;

	mov a, #0FFh	; send fourth byte dummy for receiving 69h
	lcall SPI 		;

	cjne A,#069h,enable_error ; received byte is not valid confirm one

	sjmp	Ena_OK	;success entering program mode

enable_error:
	setb	SCK			;pulse SCK to resynch
	nop
	nop
	nop
	nop
	clr	SCK			;
	nop
	nop
	nop
	nop
	djnz	bytecount,En_pg		;try this 30 times

	setb	ErrFlag		;
	clr	LED_Red		; Indicate Error
	jmp	Ena_End

Ena_OK:
	lcall delay10mS 		; wait 10 ms
	clr	LED_Grn		; Led on
Ena_End:
	ret				;
;----------------------------------------------------
;this routine sends/receives a byte to/from the device
;byte to send/receive is in acc

SPI:
	mov	bitcount,#8		;8 bits to send/rec
SPI_LP:
	push	acc			;save acc

	mov	a,b			;copy contents of 'b' into acc
	mov	c,MISO		;get incoming bit
	rlc	a			;shift left into c
	mov	b,a			;save it in b

	pop	acc			;restore a
	rlc	a			;shift left into c
	mov	MOSI,c		;send outgoing bit

	setb	SCK			;pulse SCK to TX/RX
	nop
	nop
	clr	SCK			;
	nop
	nop
	djnz	bitcount,SPI_LP	;do all bits

	mov	a,b			;return incoming byte
	ret				;in acc
;******************************************************
Lock_ISP:	
	mov	temp,#0E0h		;
	mov	bytecount,#4	;

Lock_ISP1:
	mov	a,#0ach		;
	call	SPI			;

	mov	a,temp		;lock bit in temp
	call	SPI			;	

	mov	a,#0FFh		;
	call	SPI			;
;	mov	temp2,a		;save acc

	mov	a,#0FFh		;
	call	SPI			;

	inc	temp			;this is the only way 
	lcall	Delay50mS		;to make this work!
	djnz	bytecount,Lock_ISP1

;	mov	a,temp2
;	cjne	a,#0E3h,Lock_ISP2	;
	ret

Lock_ISP2:
;	setb	ErrFlag
;	clr	LED_Red
	ret
;-----------------------------------------------
Erase_ISP:
	mov a, #0ACh 		; send first byte of erase function
	lcall SPI			;

	mov a, #080h		; send second byte
	lcall SPI			;	

	mov a, #0FFh		; send third byte (dummy byte)
	lcall SPI			;	

	mov a, #0FFh		; send fourth byte (dummy byte)
	lcall SPI			;	
;	mov	temp,a		; save returned cmd byte in temp

	lcall delay250ms 		; wait 500 milliseconds
	lcall delay250ms 
	ret
;-----------------------------------------------
Read_Sig:
	mov	temp,#00h		;signature address
	call	Rd_Sig_Byte		;temp = 00
	cjne	a,#1Eh,RS_ERR	;1st byte not 1E? 

	inc	temp			;temp = 01
	call	Rd_Sig_Byte		;next byte = 5X
	mov	chip,a

	inc	temp			;temp = 02
	call	Rd_Sig_Byte		;next byte = 06
	mov	vpp,a			;save for printing to terminal

;	mov	vpp,#0FFh		;save for printing to terminal
	ret

Rd_Sig_Byte:
	mov	a,#028h		;command byte
	call	SPI			;
	
	mov	a,temp		;address byte
	call	SPI			;
	
	mov	a,#000h		;dummy byte
	call	SPI			;	

	mov	a,#000h		;dummy byte
	call	SPI			;
	call	Time1mS
	ret
RS_ERR:
	mov	chip,#0FFh
	ret				;no error checking yet
;------------------------------------------------------
Led_Off:
	setb	LED_Grn
	setb	LED_Yel
	setb	LED_Red
	ret
;------------------------------------------------------